param (
    [string]$ip,
    [string]$user='user',
    [string]$password='password',
    [int]$is_revocable = 1,
    [string]$guid = ''
 )

function displayStatus
{
param( $status1, $status2="" )
    write-host $status1

    if( $status2.Length -ne 0 )
    {
        write-host $status2
     }
}

function displayErrorStatus
{
param( $status )
    write-host "    " $status -ForegroundColor Red
}

function findMacAddr
{
param($ip)

    # find MAC address for the hardware ID
    $macAddrQuery = (Get-WmiObject Win32_NetworkAdapterConfiguration | Where-Object{$_.IPEnabled -eq "TRUE"} | Where-Object{$_.IPAddress -contains $ip}  | Format-List MACAddress | out-string )
    $macAddr = $macAddrQuery.Split("{ }")[2]

    if( $macAddr.Length -eq 0 ) 
    {
        return -1
    }
    
    0
    $macAddr.Trim()
}

function GenerateHWID
{
param($macAddr,$scriptPath)

    if (Test-Path -Path "$scriptPath\HWID.txt")
    {
        Remove-Item "$scriptPath\HWID.txt"
    }

    try
    {
        Start-Process "taskkill" -ArgumentList "/IM hwid.exe" -Wait
        Start-Process "$scriptPath\hwID.exe" -ArgumentList "-macAddr $macAddr" -Wait
    }
    catch
    {
        write-host "Exception raised:" -ForegroundColor Red
        write-host "$($_.Exception.Message)" -ForegroundColor Red
        return -1
    }

    0
}

function getLicense
{
param($user,$password,$scriptPath)

    $hwID = (Get-Content -Path "$scriptPath\HWID.TXT" | out-string)

    # open the license web page and
    # download the license file
    $body = @{user=$user
              password=$password
              hwid=$hwID
              is_revokable=$is_revocable
              guid=$guid
              submit='Issue License' }

    $contentType = 'application/x-www-form-urlencoded' 
    $licenseOutput = "$scriptPath\license.txt"

    if (Test-Path -Path $licenseOutput)
    {
        Remove-Item $licenseOutput
    }

    try
    {
        Invoke-WebRequest -Uri "https://www.ribig.co.jp/gauthlogon/license/en/issue_license_from_script.php" -body $body -ContentType $contentType -OutFile $licenseOutput
    }
    catch
    {
        write-host "Exception raised:" -ForegroundColor Red
        write-host "$($_.Exception.Message)" -ForegroundColor Red
        return -1
    }

    0
}

function installNewLicense
{
param( $GAuthFolder,$licenseOutput)

    if (Test-Path -Path "$GAuthFolder\license.txt" )
    {
        Remove-Item "$GAuthFolder\license.txt"
    }

    Start-Process "taskkill" -ArgumentList '/IM config.exe /FI "WINDOWTITLE eq GAuthLogon*"' -Wait
    Start-Process "$GAuthFolder\config.exe" -ArgumentList "-LicenseFile $licenseOutput" -Wait

    if (Test-Path -Path "$GAuthFolder\license.txt" )
    {
        return 0
    }

    -1
}


#
# Parameters check
#
if( $ip.Length -eq 0)
{
    displayErrorStatus -status "Specify an IP with -ip parameter"
    pause
    exit 
}

if( $user.Length -eq 0)
{
    displayErrorStatus -status "Specify the account name for the license web page with -user parameter"
    pause
    exit 
}

#
# this script must be run as administrator, since licensefile must be written to Program Files folder )
# if it is not run as administrator, create a  new administrator shell and run the script in the shell
# https://stackoverflow.com/questions/7690994/powershell-running-a-command-as-administrator
#
If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
{   
    $arguments = "& '" + $myinvocation.mycommand.definition + "' -ip " + $ip + " -user " + $user + " -password " + $password

    Start-Process powershell -Verb runAs -ArgumentList $arguments

    #exit this instance run by non-administrative user
    break
}

#
# ****** Script starts here ****************
#

#the folder of his script folder
$scriptPath = (split-path -parent $MyInvocation.MyCommand.Definition)

# GAuthLogon Installation Folder
$GAuthFolder =  "$env:programfiles" + "\RiBiG\GAuthLogon"

if ( -not (Test-Path -Path $GAuthFolder) ) 
{
    displayErrorStatus -status "*** GAuthLogon Installation folder not found ***"
    pause
    exit 
}

#
# MacAddr 
#
displayStatus -status1  "Finding MAC for the specified IP: $ip"

$result = findMacAddr -ip $ip

if( $result[0] -ne 0 )
{
    displayErrorStatus -status "No MAC Address for the specified IP found"
    pause
    exit
}

$macAddr = $result[1].Trim()

displayStatus -status1 "    ---> MAC Address : $macAddr" -status2 " "

#
#  HWID
#      HWID.exe must exist in the script folder
#      HWID.exe writes HWID.txt in the same folder
#      Generate hardware ID using the MAC address
#      Hardware ID will be saved to a file named "HWID.TXT"

displayStatus -status1 "Start generating HWID"

$result = GenerateHWID -macAddr $macAddr -scriptPath $scriptPath

if( $result[0] -ne 0 )
{
    pause
    exit;
}

$hwidOutput = "$scriptPath\hwid.txt"

if ( -not (Test-Path -Path $hwidOutput) )
{
    displayErrorStatus -status1 "    --->HWID.exe did not generate hardware ID" -status2 " "
    pause
    exit
}


displayStatus -status1 "    ---> HWID has been generated." -status2 " "

#
# Web Request
#    read the hardware ID file and call the license issue web page
#    
displayStatus -status1 "Connecting to the remote license server"

$result = getLicense -user $user -password $password  -scriptPath $scriptPath

if( $result -ne 0 )
{
    pause
    exit
}

$licenseOutput = "$scriptPath\license.txt"

#
# if a license file exists, see if any error is returned
#
if (Test-Path -Path $licenseOutput)
{
    try
    {
        $result = Get-Content -Path $licenseOutput 

        if( $result.Substring(0,6).CompareTo( "ERROR:" ) -eq 0 )
        {
            $result = $result.Substring(6)

            Add-Type -AssemblyName System.Web
            $result = [System.Web.HttpUtility]::UrlDecode($result)

            $result = ConvertFrom-Json -InputObject $result

            if( $result.result -ne 0 )
            {    
                displayErrorStatus -stauts " -->" $result.error_en
                pause
            }

            exit
        }

     }
     catch
     {
        write-host "Exception raised:" -ForegroundColor Red
        write-host "$($_.Exception.Message)" -ForegroundColor Red
        pause
        exit
     }
 }

displayStatus -status1 "    --->Downloaded the license file" -status2 " "

#
# License Installation 
#
displayStatus -status1 "Installing License"

$result = installNewLicense -GAuthFolder $GAuthFolder -licenseOutput $licenseOutput

if( $result[0] -eq "0" )
{
    if (Test-Path -Path $hwidOutput)
    {
        Remove-Item $hwidOutput
    }
    displayStatus -status1 "    --->License successfully installed!" -status2 " "
}
else
{
    displayErrorStatus -status "    License installation error"
}

pause

exit


